// non ui
const Gdk = imports.gi.Gdk;
const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio
const Lang = imports.lang;
const Shell = imports.gi.Shell;
const St = imports.gi.St;
const Clutter = imports.gi.Clutter;
const Soup = imports.gi.Soup;
const Json = imports.gi.Json;
const Mainloop = imports.mainloop;
// ui stuff
const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const Panel = imports.ui.panel;
const Util = imports.misc.util;
const MessageTray = imports.ui.messageTray;
const source = new MessageTray.SystemNotificationSource();
// Text Stuff
const SITE = "http://omnimaga.org";
const APPVERSION = "0.0.1a";
const APPNAME = "Post Notifier";
const COPYRIGHT = "(c) 2011 Nathaniel 'Eeems' van Diepen";
const LIMIT = 10;
const URL = "http://omnimaga.org/feedToJson.php?limit="+LIMIT;
//const Gettext = imports.gettext.domain('gnome-shell-extensions');
//const _ = Gettext.gettext;
// Settings stuff
const APPMENU_ICON_SIZE = 22;

function HandleItem(o,e,f){
	o.fn = f;
	o.connect(e,Lang.bind(o, o.fn));
	return o;
}
function MenuItem(){
	this._init.apply(this,arguments);
}
MenuItem.prototype = {
	__proto__: PopupMenu.PopupBaseMenuItem.prototype,
	// args: Name-Label,[,Click callback[,Sub-Label]]
	_init: function(n,f,s){
		var o;
		if(typeof f === "function"){
			this.fn = f;
		}else{
			this.fn = function(){};
			o = {reactive:false};
		}
		PopupMenu.PopupBaseMenuItem.prototype._init.call(this,o);
		this.name = n;
		this.label = new St.Label({text:n});
		this.addActor(this.label);
		if(typeof s === "string"){
			this.subLabel = new St.Label({text:s});
			this.addActor(this.subLabel);
		}else if(typeof s === "object"){
			this.icon = s;
			this.addActor(this.icon);
		}
		this.connect('activate',this.fn);
	}
};
function SubMenu(){
	this._init.apply(this,arguments);
}
SubMenu.prototype = {
	__proto__: PopupMenu.PopupSubMenuMenuItem.prototype,
	// args: Name-label[,Children]
	_init: function(n,c){
		PopupMenu.PopupSubMenuMenuItem.prototype._init.call(this,n);
		this._triangle = new St.Label({text:''});
		this.name = n;
		this._ITEMS = c;
		this.menuItems = [];
		this._createMenu();
	},
	_createMenu: function() {
		var i;
		//create the menu here
		for(i=0;i<this._ITEMS.length;i++){
			this.menuItems[i] = this.menu.addMenuItem(this._ITEMS[i]);
		}
	}
};
function PostObj(){
	this._init.apply(this,arguments);
}
PostObj.prototype = {
	_init: function(post){
		this.title = post.get_string_member('title');
		this.link = post.get_string_member('link');
		this.pubDate = post.get_string_member('pubDate');
	}
};
function PostNotify() {
	this._init.apply(this, arguments);
}
PostNotify.prototype = {
	__proto__: PanelMenu.SystemStatusButton.prototype,
	_init: function() {
		PanelMenu.SystemStatusButton.prototype._init.call(this, 'emblem-default');
		this._last = null;
		this._interval = 1;
		this._enabled = true;
		this._post = Date.parse(new Date());
		PostNotifyObj = this;
		this._createMenu();
		this._loop();
	},
	connect: function(t,f){
		this.actor.connect(t,f);
	},
	_loop: function(){
		if(this._enabled){
			this._check_for_posts();
			Mainloop.timeout_add_seconds(60*this._interval, Lang.bind(PostNotifyObj, PostNotifyObj._loop));
		}
	},
	_ITEMS: [
		//new MenuItem("Test Alert",function(){notify('DEBUG','Hello, World!');}),
		new MenuItem("Check for posts",function(){
			PostNotifyObj._check_for_posts()
		},new St.Icon({
			icon_type: St.IconType.FULLCOLOR,
			icon_size: Main.panel.button.get_child().height - 4,
			icon_name: 'view-refresh-symbolic',
		})),
		new SubMenu("Settings",[
			HandleItem(new PopupMenu.PopupSwitchMenuItem("Enabled",true),"activated",function(){
				PostNotifyObj._enabled = !PostNotifyObj._enabled;
				if(PostNotifyObj._enabled){
					PostNotifyObj._loop();	
				}
			}),
			new MenuItem("Interval",undefined,"1 minute"),
			HandleItem(new PopupMenu.PopupSliderMenuItem(0),'value-changed',function(){
				PostNotifyObj.setInterval(Math.round(this.value*10));
				PostNotifyObj._ITEMS[1]._ITEMS[1].subLabel.set_text(PostNotifyObj._interval+" minute");
			})
		]),
		new SubMenu("About",[
			new MenuItem("Site",function(){
				openURL(SITE);
			},SITE),
			new PopupMenu.PopupSeparatorMenuItem(),
			new MenuItem(APPNAME),
			new MenuItem("Version:",null,APPVERSION),
			new MenuItem("Copyright:",null,COPYRIGHT)
		])
	],
	setInterval: function(v){
		this._interval = Math.round(v+1);
	},
	_createMenu: function() {
		var i;
		//create the menu here
		for(i=0;i<this._ITEMS.length;i++){
			this.menu.addMenuItem(this._ITEMS[i]);
		}
	},
	_notify: function(tt,t,l){
		let notification = new MessageTray.Notification(source,tt,t,{icon:new St.Icon({icon_name:'emblem-default',icon_type:St.IconType.SYMBOLIC,icon_size:source.ICON_SIZE})});
		if(typeof l !== "undefined"){
			notification.__link = l;
		}else{
			notification.__link = false;
		}
		notification.setResident(true);
		notification.connect("clicked",function(a,e){
			if(a.__link){
				PostNotifyObj._openURL(a.__link);
			}
			a.destroy();
		});
		source.notify(notification);
	},
	_openURL: function(url){
		try {
			Gio.app_info_launch_default_for_uri(url, global.create_app_launch_context());
		} catch (e) {
			Util.spawn(['gvfs-open', url]);
		}
	},
	_loadURL: function(url) {
		var session = new Soup.SessionSync();
		var message = Soup.Message.new('GET', url);
		stat = session.send_message(message);
		if(stat==200){
			return message.response_body.data;
		}else{
			return false;
		}
	},
	_loadJSON: function(url){
		jp = new Json.Parser();
		t = this._loadURL(url);
		if(t){
			jp.load_from_data(t, -1);
			return jp.get_root().get_object();
		}else{
			return false;
		}
	},
	_check_for_posts: function(){
		this._json = loadJSON(URL);
		if(this._json){
			let a = [];
			this._json = this._json.get_array_member('posts');
			var T = false;
			for(var i=0; i<this._json.get_length();i++){
				post = new PostObj(this._json.get_object_element(i));
				if(this._post < Date.parse(post.pubDate)){
					this._notify("New Post",post.title,post.link);
					a.push(Date.parse(post.pubDate));
					T = true;
					
				}
			}
			this._a = a;
			if(T){
				this._post = Math.max.apply( Math, a );
			}
		}
	},
	source:source
};
PostNotifyObj = null;
function notify(tt,t,l){
	PostNotifyObj._notify(tt,t,l);
}
function openURL(u){
	PostNotifyObj._openURL(u);
}
function loadURL(u){
	return PostNotifyObj._loadURL(u);
}
function loadJSON(u){
	return PostNotifyObj._loadJSON(u);
}
function main(meta) {
	Panel.STANDARD_TRAY_ICON_ORDER.unshift('post-notify');
	Panel.STANDARD_TRAY_ICON_SHELL_IMPLEMENTATION['post-notify'] = PostNotify;
	source.title = "PostNotify";
	source._lastNotificationRemoved = function(){};
	source._setSummaryIcon(new St.Icon({icon_name:'emblem-default',icon_type:St.IconType.SYMBOLIC,icon_size:source.ICON_SIZE}));
	source.destroy = function(){
		for(var i in source.notifications){
			source.notifications[i].setResident(false);
		}
		source.destroyNonResidentNotifications()
	};
	Main.messageTray.add(source);
	//PostNotifyObj.connect("","");
}

